#include <string.h>
#include "types.h"
#include "proto.h"
#include "read.h"
#include "menu.h"
#include "select.h"
#include "matrix.h"

/* program global variables */
         char   errorstring[1025];
         string progname;

/* local protoypes */
int main (int argc, string argv[], string envp[]);
static void usage(const char product[], const char version[], 
     const string progname);
static string basename(string command);

/* exported global variables */
   const char product[]   = "IR-STAT-PAK";
   const char version[]   = "1.02";
   const char copyright[] = "Copyright (c) 1995 by James Blustein";

int main(int argc, string argv[], string envp[]) 
{
          const    char          proc[]    = "main";
                   chose_t       participant_s,  query_s;
                   Matrix        *precision_mat[NUMofPRECISION], 
                                 *interpolate_mat[NUMofINTERPOL], 
                                 *numdoc_mat[NUMofNUMDOCS];
          register bool          okay = TRUE;
                   unsigned long read_result;
   extern const char             product[];
   extern const char             version[];
   extern const char             copyright[];
   extern const string           typename[];   
   extern const string *         valname[];   
   static const string           precname[] = {"Average Precision", 
                                               "R-precision"};

   printf("%s %s\n", product, copyright);
   /* initialization */
   progname = basename(argv[0]);

   if (argc > 2) {
      usage(product, version, progname);
      return (2); 
   }

   (void)select_init();
   s_init(&participant_s, "Participant", MAX_PARTICIPANT, PARTI_NAME_LEN);
   s_init(&query_s,       "Query",       NUMofQUERIES,    QUERY_NAME_LEN);

   okay = 
    okay                                                                      
     &&
    mat_3D_init(precision_mat, NUMofPRECISION, NUMofQUERIES, MAX_PARTICIPANT, \
                NULL, precname)
     &&
    mat_3D_init(numdoc_mat, NUMofNUMDOCS, NUMofQUERIES, MAX_PARTICIPANT,      \
                typename[NumDoc], valname[NumDoc])
     &&
    mat_3D_init(interpolate_mat, NUMofINTERPOL, NUMofQUERIES, MAX_PARTICIPANT,\
                typename[Interpol], valname[Interpol]);
   if (!okay) {
      swerr(proc,"Not enough memory for data matrices!", 1);  /* NOTREACHED */
   }

   if (2 == argc) {
      read_result = read_data(argv[1], precision_mat, interpolate_mat, \
                              numdoc_mat, &participant_s, &query_s);

      if (0UL == read_result) {
         fputs("\nNAME:\n   ", stderr);
         usage(product, version, progname);
         if (FALSE == wait_for_enter_key()) {
            exit (1); /* NOTREACHED */
	 }
      }
   }

   return(main_menu(precision_mat, interpolate_mat, numdoc_mat, \
                    &participant_s, &query_s)
          ?0 :1);
} /* main() */


void usage(const char   product[],
           const char   version[],
           const string progname) 
{
      fprintf(stderr, "%s version %s\n\n", product, version);
      fprintf(stderr, "DESCRIPTION:\n   ");
      fprintf(stderr, "%s will perform a statistical analysis of TREC " \
                      "trial outputs.\n\n",
                      product);
      fprintf(stderr, "USAGE:\n       %s\n   or\n       %s datafile \n\n", \
                      progname, progname);
      fprintf(stderr, "     where `datafile' is the name of the file of " \
                      "TREC data in the %s\n"                             \
                      "                      input format.\n",            \
                      product);
      fprintf(stderr, "     N.B.: The program has online documentation that " \
                      "details the format.\n");
} /* usage() */


/************
 * basename *
 ************
  PURPOSE: Returns the filename portion of the input parameter, e.g. 
    basename("../foo/bar/quux.txt") would return "quux.txt".
  PRE: `command' is a Unix filename with an optional path
  POST: A string representing the filename portion of the input paramater 
      has been returned if space could be allocated on the heap for it.  
      Otherwise the input parameter was returned.
*/
string basename(string command)  /* Unix filename as a string */
{
   string name = strdup(command);
   string basename;

   if (name == NULL) {
      name = command;
   } else {
      basename = strrchr(name, '/');
      if (basename != NULL) {
         name = basename + 1;
      }
   }

   return(name);
 } /* basename() */


