/* matrix. c */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <malloc.h>
#include "types.h"
#include "proto.h"
#include "matrix.h"
#include "select.h"

/* local protypes */
static Matrix * mat_init(const matrix_dim row_count, 
          const matrix_dim col_count, const string title, const string level);

/*                                                                         *\
   -----------------------------------------------------------------------
                  Matrix (wrapper for 2D array of reals)
   -----------------------------------------------------------------------
\*                                                                         */

#define MAX_ROWS NUMofQUERIES
#define MAX_COLS MAX_PARTICIPANT

Matrix * mat_init(const matrix_dim row_count,
                  const matrix_dim col_count,
                  const string     title,
                  const string     level) 
{
/*
    Dynamically allocates space for a maximal matrix and sets its row and
column sizes to the paramaters.  To resize the matrix use mat_resize(),
mat_col_inc() and mat_col_dec().
*/

   extern char     errorstring[];
          char     proc[] = "mat_init";
          Matrix * mat    = NULL;
   
   if ((MAX_COLS >= col_count) && (MAX_ROWS >= row_count)) {
       if (NULL != (mat = (Matrix *)malloc(sizeof(Matrix)))) {
          mat->row_count = row_count;
          mat->col_count = col_count;
       }
       if (NULL != level) {
          strcpy(mat->level, level);
	} else {
           mat->level[0] = '\0';
	}
       if ((NULL != (mat->title = title)) && 
           (NULL == (mat->title = strdup(title)))) {
             sprintf(errorstring, "not enough memory for matrix title \"%s\"",
                                  title);
             swerr(proc, errorstring, 0);
       }
   } else {
      sprintf(errorstring, "request for too large a matrix (" \
                           MAT_DIM_SPEC" x "MAT_DIM_SPEC")",  \
                           col_count, row_count);
      swerr(proc, errorstring, 0);
   }
   return (mat);
} /* mat_init() */


bool mat_3D_init(      Matrix *   threeD[],
                 const elem_t     depth,
                 const matrix_dim row_count,
                 const matrix_dim col_count,
                 const string     title,
                 const string     levels[])
{
   register matrix_dim i;
   register bool       okay = TRUE;

   for (i=0; okay && (i < depth); i++) {
      okay =                                                                  \
             okay && (NULL != (threeD[i] = mat_init(row_count, col_count,     \
                                                    title, (levels)?levels[i] \
						                   :NULL)));
   }
   return (okay);
} /* mat_3D_init() */


void copy_to_submatrix(      Matrix *   submatrix[],
                             Matrix *   original[],
                       const chose_t *  row_select,
                       const chose_t *  col_select,
                       const chose_t *  depth_select) 
{
            char       proc[] = "copy_to_submatrix";
   register matrix_dim col, row, newrow, newcol;
   register elem_t     depth, newdepth;
   const    matrix_dim max_rows=s_count(row_select);
   const    matrix_dim max_cols=s_count(col_select);
   const    matrix_dim max_depth=s_count(depth_select);

   for (depth=newdepth=0; newdepth < max_depth; depth++) {
      if (s_value(depth_select, depth)) {
         if (mat_resize(submatrix[newdepth], max_rows, max_cols)) {
            if (NULL != (submatrix[newdepth]->title = original[depth]->title) \
              &&                                                              \
                NULL == (submatrix[newdepth]->title                           \
                         = strdup(original[depth]->title))){
               swerr(proc, "not enough memory to copy names into submatrix",0);
	    }
            strcpy(submatrix[newdepth]->level, original[depth]->level);
            for (col=newcol=0; newcol < max_cols; col++) {
               if (s_value(col_select, col)) {
                  for (row=newrow=0; newrow < max_rows; row++) {
                     if (s_value(row_select, row)) {
                        mat_set(submatrix[newdepth],newrow,newcol, \
                                mat_value(original[depth],row,col));
                        newrow++;
                     }
                  }
                  newcol++;
               }
            }
         } else {
            swerr(proc, 
                  "can't resize submatrix! (this should never happen)", 1);
         }
         newdepth++;
      }
   }
} /* copy_to_submatrix() */


bool mat_chosen(const chose_t *     parti_s,
                const chose_t *     query_s,
                      chose_t *     level_s[],
                      Matrix **     all_data[],
                      Matrix **     chosen[],
                      string        parti_name[],
                      string        query_name[],
                      TransFuncType transform_to_apply)
{
            char       proc[] = "mat_chosen";
   register matrix_dim type;

   if (((NULL == parti_name) || (make_name_list(parti_name, parti_s))) && \
       ((NULL == query_name) || (make_name_list(query_name, query_s)))) {
      for (type=0; type < NUMofTYPES; type++) {
         copy_to_submatrix(chosen[type], all_data[type], query_s, parti_s, \
                           level_s[type]);
         if (NULL != transform_to_apply) {
            mat_forall(chosen[type], transform_to_apply, \
                       s_count(level_s[type]));
         }
      }
      return(TRUE);
   } else {
      swerr(proc, "Can't make list of selected data!", 0);
      return(FALSE);
   }
} /* mat_chosen() */


string mat_title(const Matrix * mat) 
{
   return (mat->title);
} /* mat_title() */


string mat_level(const Matrix * mat) 
{
   return (mat->level);
} /* mat_level() */


bool mat_resize(      Matrix *   mat,
                const matrix_dim new_rows,
                const matrix_dim new_cols) 
{
   if ((MAX_COLS >= new_cols) && (MAX_ROWS >= new_rows)) {
      mat->row_count = new_rows;
      mat->col_count = new_cols;
      return(TRUE);
   } else {
      return(FALSE);
   }
} /* mat_resize() */


bool inc_mat_col(Matrix * mat) 
{
   if (MAX_COLS > mat->col_count) {
      (mat->col_count)++;
      return (TRUE);
   } else {
      return (FALSE);
   }
} /* inc_mat_col() */


bool dec_mat_col(Matrix * mat) 
{
   if ( 0 < mat->col_count) {
      (mat->col_count)--;
      return (TRUE);
   } else {
      return (FALSE);
   }
} /* dec_mat_col() */


real mat_value(const Matrix *   mat,
               const matrix_dim row,
               const matrix_dim col) 
{
   extern char errorstring[];
          char proc[] = "mat_value";

   if ((row <= mat->row_count) && (col <= mat->col_count)) {
      return(mat->matrix[col][row]);
   } else {
      sprintf(errorstring,                                      \
              "attempt to read uninitialized value "            \
              "(row="MAT_DIM_SPEC",col="MAT_DIM_SPEC")\n"       \
              "matrix is 0.."MAT_DIM_SPEC" by 0.."MAT_DIM_SPEC, \
              row, col, mat->row_count, mat->col_count);
      swerr(proc, errorstring,0);
      if (mat->title) {
         sprintf(errorstring,                          \
                 "matrix has title \"%s\"", mat->title);
         swerr(proc, errorstring, 0);
      } else {
         swerr(proc, "matrix has no title", 0);
      }

      #ifdef UWOCSD
         return ((real)signaling_nan(0L));
      #else
         return ((real)-1.0);
      #endif
   }
} /* mat_value() */


bool mat_set(      Matrix *   mat,
             const matrix_dim row,
             const matrix_dim col,
             const real       value) 
{
   extern char errorstring[];
          char proc[] = "mat_set";

   if ((row <= mat->row_count) && (col <= mat->col_count)) {
      mat->matrix[col][row] = value;
      return (TRUE);
   } else {
      sprintf(errorstring,                                 \
              "attempt to write to unreserved space (row=" \
              MAT_DIM_SPEC",col="MAT_DIM_SPEC")",          \
              row, col);
      swerr(proc, errorstring, 0);
      return (FALSE);
   }
} /* mat_value() */


matrix_dim num_rows(const Matrix * mat) 
{
   return(mat->row_count);
} 


matrix_dim num_cols(const Matrix * mat) {
   return(mat->col_count);
} /* num_cols() */


/*                                                                         *\
   -----------------------------------------------------------------------
                      Sums and Averages with Matrices
   -----------------------------------------------------------------------
\*                                                                         */

real sumcol(      Matrix *   mat,
            const matrix_dim column) 
{
   register matrix_dim row;
   register real       sum = 0.0;

   for(row=0; row < mat->row_count; row++) {
/*
      sum += mat->matrix[column][row];
*/
      sum += mat_value(mat,row,column);
   }
   return (sum);
} /* sumcol() */


real sumrow(      Matrix *   mat,
            const matrix_dim row) 
{
   register matrix_dim column;
   register real       sum = 0.0;

   for(column=0; column < mat->col_count; column++) {
      sum += mat->matrix[column][row];
   }
   return (sum);
} /* sumrow() */


real sum3Dcol(      Matrix *   arr[],
              const matrix_dim column,
              const matrix_dim howdeep) 
{
   register matrix_dim depth;
   register real       sum = 0.0;

   for (depth=0; depth < howdeep; depth++) {
      sum += sumcol(arr[depth], column);
   }
   return (sum);
} /* sum3Dcol() */


real sum3Drow(      Matrix *   arr[],
              const matrix_dim row,
              const matrix_dim howdeep) 
{
   register matrix_dim depth;
   register real       sum = 0.0;

   for (depth=0; depth < howdeep; depth++) {
      sum += sumrow(arr[depth], row);
   }
   return (sum);
} /* sum3Drow() */


real sumdepth(      Matrix *   arr[],
              const matrix_dim row,
              const matrix_dim col,
              const matrix_dim maxdepth) 
{
   register matrix_dim depth;
   register real       sum = 0.0;

   for (depth=0; depth < maxdepth; depth++) {
      sum += arr[depth]->matrix[col][row];
   }
   return (sum);
} /* sumdepth() */


/*                                                                         *\
   -----------------------------------------------------------------------
                                  For All
   -----------------------------------------------------------------------
\*                                                                         */

void mat_forall(      Matrix *      mat[],
                      TransFuncType (* func),
                const matrix_dim    max_depth)
{
   register matrix_dim  depth, row, col;
   const    matrix_dim  max_row = num_rows(mat[0]);
   const    matrix_dim  max_col = num_cols(mat[0]);

   if (NULL != func) { /* sanity check */
      for (depth=0; depth < max_depth; depth++) {
         for (row=0; row < max_row; row++) {
            for (col=0; col < max_col; col++) {
               func(&(mat[depth]->matrix[col][row]));
	    }
         }
      }
   }
} /* mat_forall() */

