#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "types.h"
#include "proto.h"
#include "plot.h"
#include "print.h"
#include "select.h"

/* global variables imported from other modules */
   extern const char       product[];  /* from main.c */
   extern const char       version[];  /* from main.c */

/* global variables used in this module only */
   static const real   HISTO_GAP = 1.0;
   static const real   HISTO_BAR = 2.0;
   static const string JGRAPH    = "\"/usr/local/bin/jgraph\"";
   static const string GHOSTVIEW = "\"/usr2/X11/bin/ghostview\" " \
                                   "-magstep -2 -nocenter -geometry +0+0";
   /* N.B.: absolute path names! */
   static const float graph_width  =  8.0F; /* must be in inches */
   static const float graph_height = 10.0F; /* must be in inches */

#define MAX_PLOT_NAME_LEN (254)

/* local prototypes */
static real make_bar(FILE * outfile, const string curvename, 
          const string fillpattern, const bool second_pt, const real inc, 
          const unsigned numpts, const real data[]);
static bool make_histo(const string plottitle, const string plotname, 
          const string dataname[NUMofPRECISION], real ** average, 
          const matrix_dim numpts, const string label[], FILE * destname);
static bool make_line(FILE * outfile, real ** data,
          const string partiname[], const matrix_dim maxparti);

bool graph_make(const bool       histogram,  /* */
                const string     datatitle,  /* Parti, Query, or P/Q */
                      real *     data[],     /* */
                const matrix_dim numpts,     /* size of data[] */
                const string     label[],    /* names of bars, e.g. Parti n */
                const string     barname,    /* if 1 bar -> which measure */
                      int *      err) /* passes in #bars in histogram,
                                         passes back exit status */
{
                char       proc[] = "graph_make";
                FILE *     dest   = NULL;
                char       destname[MAXFNAMELEN + 1];
                char       title[MAX_PLOT_NAME_LEN + 1];
   static const ScreenSize notice_width = 80;
   extern       int        errno;
   extern const float      graph_width; 
   extern const float      graph_height;
            
   start_menu();
   PrintTitle(stdout, (histogram)?"Precision histogram" \
                                 :"Recall-Precision plot", notice_width);
   printf("This option does not produce plots on its own.  Instead, " \
          "it uses the jgraph ");
   NewLine(1, stdout);
   fputs("program.  You will need to supply the names of two files: " \
         "one for the jgraph ", stdout);
   NewLine(1, stdout);
   fputs("commands and one for the Postscript(tm) output it will " \
         "produce.", stdout);
   NewLine(2, stdout);

   if (NULL != (dest=OutputFile(destname, MAXFNAMELEN))) {
      GetStringFromUser(title, MAX_PLOT_NAME_LEN, "Enter title for figure ");
      chop(title);
      fprintf(dest,   \
              "(* jgraph data and command file prepared by %s " \
              "version %s *)\n", product, version);
      if (histogram) {
         extern const string typename[]; 
                      bool   both = (2 == *err);
                      string legend[NUMofPRECISION];
            
         if (both) {
            legend[AvPre] = typename[AvPre];
            legend[RPre]  = typename[RPre];
	 } else {
            legend[0] = barname;
            legend[1] = NULL;
	 }
         if (make_histo(title,datatitle,legend,data,numpts,label,dest)) {
            *err = (close_file(proc, destname, dest)?no_problem:closing);
	 } else {
            *err = sub;
	 }
      } else {
         fprintf(dest, "newgraph\n"     \
                       "legend bottom\n"\
                       "title : %s\n\n" \
                       "yaxis\n "       \
                       "   size %g\n",  \
                       title, graph_height);
         fprintf(dest, "   hash 0.1 min 0 max 1 label : Precision\n" \
                       "   shash 1 precision 1 no_auto_hash_labels\n");
         fprintf(dest, "   hash_label at  0 : 0\n"
                       "   hash_label at .1 : 0.1\n"
                       "   hash_label at .2 : 0.2\n"
                       "   hash_label at .3 : 0.3\n"
                       "   hash_label at .4 : 0.4\n"
                       "   hash_label at .5 : 0.5\n"
                       "   hash_label at .6 : 0.6\n"
                       "   hash_label at .7 : 0.7\n"
                       "   hash_label at .8 : 0.8\n"
                       "   hash_label at .9 : 0.9\n"
                       "   hash_label at  1 : 1\n");
         fprintf(dest, "xaxis\n"        \
                       "   size %g\n"
                       "   hash 0.1 min 0 max 1 label : Recall\n"     \
                       "   shash 1 precision 2 no_auto_hash_labels\n",\
                       graph_width);
         fprintf(dest, "   hash_label at  0 : 0.00\n"
                       "   hash_label at .1 : 0.10\n"
                       "   hash_label at .2 : 0.20\n"
                       "   hash_label at .3 : 0.30\n"
                       "   hash_label at .4 : 0.40\n"
                       "   hash_label at .5 : 0.50\n"
                       "   hash_label at .6 : 0.60\n"
                       "   hash_label at .7 : 0.70\n"
                       "   hash_label at .8 : 0.80\n"
                       "   hash_label at .9 : 0.90\n"
                       "   hash_label at  1 : 1.00\n");
         if (make_line(dest, data, label, numpts)) {
            *err = (close_file(proc, destname, dest)?no_problem:closing);
	 } else {
            *err = sub;
         }
      }
   } else {
      *err = user_abort;
   }

   if (no_problem == *err) {
      char psname[MAXFNAMELEN + 1];
      char system_str[3 * MAXFNAMELEN + 6];

      GetStringFromUser(psname, MAXFNAMELEN, \
                        "Enter name for postscript file ");
      chop(psname);
      if (psname[0]) {
         int    sys_stat;

         sprintf(system_str, "%s \"%s\" > \"%s\"", \
                 JGRAPH, destname, psname);
         sys_stat = system(system_str);
         if (0 == sys_stat) {
            if (NULL != getenv("DISPLAY")) {
               sprintf(system_str, "%s \"%s\" &", GHOSTVIEW, psname);
               (void)system(system_str);
	    } else {
               fputs("I do not detect a correctly configured X Window " \
                     "display, so I can not", stdout);
               NewLine(1, stdout);
               fputs("display the graph.  You can still use the Postscript", \
                     stdout);
               NewLine(1, stdout);
               fputs("and jgraph files.", stdout);
               NewLine(1, stdout);
               end_display();
	    }
         } else {
            NewLine(1, stdout);
            fputs("Something went wrong when I tried to create the " \
                  "Postscript file.", stdout);
            NewLine(1, stdout);
            fputs("Please have the problem fixed and try again.", stdout);
            NewLine(1, stdout);
            sprintf(errorstring, "system() returned %d (call string " \
                                 "follows)", sys_stat);
            swerr(proc, errorstring, 0);
            errno = 0;
            swerr(proc, system_str, 0);
	 }
      } else {
         fputs("No Postcript file created.", stdout);
         NewLine(1, stdout);
      }
      return(TRUE);
   } else {
      return(FALSE);
   }
} /* graph_make() */
	   

bool make_histo(const string     plottitle,  /* title for entire graph */
                const string     x_label,    /* label for x-axis */
                const string     dataname[NUMofPRECISION],
                      real **    averages, /* data to plot */
                const matrix_dim numpts,   /* how many points in averages[] */
                const string     label[MAX_MATRIX], 
                      FILE *     dest)
{
   extern const float graph_width; 
   extern const float graph_height;
          const bool     both = (NULL != dataname[NUMofPRECISION - 1]);
          const string   patt[NUMofPRECISION] = {"fill .4", "fill .9"};
                unsigned point;
                real     pos, /* postion at which to print label */
                         inc; /* increment to add between values of pos */

   fprintf(dest, "newgraph\n  legend bottom\n  title : %s\n", plottitle);
   fprintf(dest, "  yaxis\n"
                 "    size %g\n"
                 "    hash 0.1 min 0 max 1 label : Precision\n"  \
                 "    shash 1 precision 1 no_auto_hash_labels\n",\
                 graph_height);
   fprintf(dest, "    hash_label at  0 : 0\n"
                 "    hash_label at .1 : 0.1\n"
                 "    hash_label at .2 : 0.2\n"
                 "    hash_label at .3 : 0.3\n"
                 "    hash_label at .4 : 0.4\n"
                 "    hash_label at .5 : 0.5\n"
                 "    hash_label at .6 : 0.6\n"
                 "    hash_label at .7 : 0.7\n"
                 "    hash_label at .8 : 0.8\n"
                 "    hash_label at .9 : 0.9\n"
                 "    hash_label at  1 : 1\n");
         if (both) {
            inc = (HISTO_BAR + 3.0 * HISTO_GAP);
            (void)make_bar(dest, dataname[0], patt[0], FALSE, inc, \
                           numpts, averages[0]);
            pos = make_bar(dest, dataname[1], patt[1], TRUE,  inc, \
                           numpts, averages[1]);
	 } else {
            inc = (HISTO_BAR + HISTO_GAP);
            pos = make_bar(dest, dataname[0], patt[0], FALSE, \
                           inc, numpts, averages[0]);
         }

         fprintf(dest, "  xaxis  min 0 max %5.1f\n", pos - inc + HISTO_BAR);
         fprintf(dest, "    label : %s\n" \
                       "    size %g\n"
                       "    no_auto_hash_marks no_auto_hash_labels\n" \
                       "    hash_labels rotate -90 vjc hjl\n",        \
                       x_label, graph_width);
         if (numpts > 25) {
            fprintf(dest, "   fontsize 4\n");
	 }
         
         /* x-axis labels */
         pos = (!both) ?(HISTO_GAP + 0.5 * HISTO_BAR)\
                       :(HISTO_GAP + HISTO_BAR);
         inc = (!both) ?(HISTO_GAP + HISTO_BAR)      \
                       :(HISTO_BAR + 3.0 * HISTO_GAP);
         for(point = 0; point < numpts; point++, pos += inc) {
            fprintf(dest, "      hash_label at %5.1f : %s\n", \
                          pos, label[point]);
         }
         return(TRUE);
} /* make_histo() */


real make_bar(      FILE *   outfile,
              const string   barname,
              const string   fillpattern,
              const bool     second_pt,
              const real     inc,
              const unsigned numpts,
              const real     data[])     {

      real     pos = 2.0 * (real)HISTO_GAP;
      unsigned point;

      fprintf(outfile, "  newcurve\n");
      fprintf(outfile, "    label : %s\n", barname);
      fprintf(outfile, "    marktype xbar %s\n    marksize %f\n    pts\n", \
                       fillpattern, HISTO_BAR);
      if (second_pt) { 
         pos += HISTO_BAR; 
      }
      for (point = 0; point < numpts; point++, pos += inc) {
         fprintf(outfile, "     %5.1f %#g\n", pos, data[point]);
      }
      return(pos);
} /* make_bar() */


bool make_line(      FILE *     outfile,
                     real **    data,
               const string     partiname[],
               const matrix_dim maxparti) {

   extern   const      string *   valname[];
            const      char       proc[] = "make_line";
            register   matrix_dim parti,
                                  level;
            const      string     linetype[] = {\
                                     "solid",   "dotted", "dashed", "longdash",
                                     "dotdash", "dotdotdash", 
                                     "dotdotdashdash"};
            const      string     marktype[] = {\
                                     "circle", "box",   "triangle", "diamond", 
                                     "x",      "cross", "ellipse"};
            const      unsigned   maxmark= sizeof marktype/sizeof marktype[0];
            const      unsigned   maxline= sizeof linetype/sizeof linetype[0];
                       bool       okay = TRUE;

   for (okay=TRUE,parti=0; okay && (parti < maxparti); parti++) {
      okay = fprintf(outfile, "  newcurve\n"                  \
                              "    label : %s\n"              \
                              "    linetype %s marktype %s\n" \
                              "    pts\n",                    \
                              partiname[parti],               \
                              linetype[parti < maxmark        \
                                       ?0 :(parti % maxline)],\
                              marktype[parti % maxmark]);
      for (level=0; okay && (level < NUMofINTERPOL); level++) {
         okay = okay && \
                fprintf(outfile, "      %s %#g\n", \
                                 valname[Interpol][level], data[level][parti]);
      }
   }
   if (!okay) {
      swerr(proc, "writing to jgraph command file", 0);
   }
   return(okay);
} /* make_line() */
