#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "types.h"
#include "proto.h"
#include "boolarr.h"
#include "print.h" /* for start_menu() */
#include "select.h"
#ifndef UWOCSD
   char *strdup(const char *s1);
#endif

/*                                                                         *\
   -----------------------------------------------------------------------
                      Select (wrapper for bitvector)
   -----------------------------------------------------------------------
\*                                                                         */

bool select_init(void) 
{
   static bool first_time = TRUE;

   if (first_time) {
      (void)ba_init();
      first_time = FALSE;
      return (TRUE);
   } else {
      return (FALSE);
   }
}/* select_init() */


bool s_init(      chose_t * select,
            const string    title,   /* name of the data it will represent */
            const elem_t    max,     /* maximum number of elem */
            const elem_t    max_len) /* longest allowable name */
{
   register unsigned int i;

   select->size = 0;
   select->max = max;
   select->max_len = max_len;
   if (NULL == title) {
      select->title = NULL;
   } else {
      if (NULL == (select->title = strdup(title))) {
         return (FALSE);
      }
   }
   if (NULL == (select->selected = ba_new(max)) ||
      (NULL == (select->name = calloc((size_t)max, sizeof(string))))) {
      return (FALSE);
   }
   ba_all_assign(select->selected, max, FALSE);
   for(i=0; i < max; i++) {
      if (NULL != (select->name[i] = 
                   (string)calloc((size_t)max_len+1, sizeof(char)))) {
         strcpy(select->name[i], "");
      } else {
         return (FALSE);
      }
   }
   return (TRUE);
} /* s_init() */


void s_destroy(chose_t * select)
{
   register elem_t i;

   if (NULL != select->title) {
      free(select->title);
   }
   if (NULL != select->selected) {
      free(select->selected);
   }
   for (i=0; i < select->max; i++) {
      if (NULL != select->name[i]) {
         free(select->name[i]);
      }
   }
   free(select);
} /* s_destroy() */


elem_t s_count(const chose_t *select)
{
   return (ba_count(select->selected, select->size));
} /* s_count() */


string s_title(const chose_t * select)
{
   return(select->title);
} /* s_title() */


string s_name(const chose_t * select,
              const elem_t    elem,
                    bool *    err) 
{
   *err = (elem < select->size);
   return (*err)?(select->name[elem]):NULL;
} /* s_name() */


bool s_remove(chose_t * where) 
{
   if (where->size > 0) {
      where->size--;
      /*where->name[where->size][0] = '\0';*/
      strncpy(where->name[where->size], "**REMOVED**", where->max_len);
      where->name[where->size][where->max_len] = '\0';
      ba_assign(where->selected, where->size, FALSE); /* canonical form */
      return(TRUE);
   } else {
      return(FALSE);
   }
} /* s_remove() */


bool s_append(const string    name,
                    chose_t * where,
              const bool      selected) 
{
   if (where->size < where->max) {
      strncpy(where->name[where->size], (NULL != name)?name:"", \
             (size_t)where->max_len);
      where->name[where->size][where->max_len] = '\0';
      ba_assign(where->selected, where->size, selected);
      where->size++;
      return(TRUE);
   } else {
      return(FALSE);
   }
} /* s_append() */


elem_t s_size(const chose_t * select) 
{
   return (select->size);
} /* s_size() */


bool s_value(const chose_t * select,
             const elem_t    elem) 
{
   return (ba_value(select->selected, elem)) ?TRUE: FALSE;
} /* s_value() */


bool s_all_are_on(const chose_t * select) 
{
   return (select->size == s_count(select));
} /* s_all_are_on() */


bool s_assign(const chose_t * select,
              const elem_t    elem,
              const bool      value) 
{
   if (elem < select->size) {
      ba_assign(select->selected, elem, value);
      return (TRUE);
   } else {
      return (FALSE);
   }
} /* s_assign() */


void s_toggle(      chose_t * select,
              const elem_t    elem) 
{
   ba_toggle(select->selected, elem);
} /* s_toggle() */


void s_all_select(      chose_t * select,
                  const bool      value) 
{
     ba_all_assign(select->selected, select->size, value);
} /* s_all_select() */


/*                                                                         *\
   -----------------------------------------------------------------------
                                Select Menu
   -----------------------------------------------------------------------
\*                                                                         */

bool s_select_menu(chose_t * select)
{
            const char           proc[] = "s_select";
   extern   const char           PROMPT[];
   register       elem_t         i;
   register       bool           change = FALSE;
                  elem_t         size = s_size(select);
                  unsigned short choice;

   /*CONSTCOND*/
   while (TRUE) {
      start_menu();
      printf("Select %s\n", select->title);
      for (i=0; i < size; i++) {
         printf("   %c %2lu.  %*s\n",                \
               (s_value(select, i))?on :off, i+1,    \
               (int)select->max_len, select->name[i]);
      }
      printf("     %2lu.  select all\n", ++i);
      printf("     %2lu.  do not use %s\n", ++i, select->title);
      printf("     %2lu.  return to previous menu\n", ++i);

      choice = getchoice(1,size+3,PROMPT,0);
      if (choice != 0) {
         choice--;
         if (choice < size) {
            change = TRUE;
            s_toggle(select, (elem_t)choice);
	 } else {
            choice -= size;
            switch (choice) {
               case 0: if (size != s_count(select)) {
                          change = TRUE;
		       }
	               s_all_select(select, TRUE);
                       break;
	       case 1: if (0 != s_count(select)) {
                          change = TRUE;
	               }
	               s_all_select(select, FALSE);
                       break;
	       case 2: return (change);
	       default:
                       swerr(proc, "impossible choice", 1);
	    } /* switch */
	 }
      }
   } /* while */
   /* NOTREACHED */
} /* s_select() */



/*                                                                         *\
   -----------------------------------------------------------------------
                                 Name list
   -----------------------------------------------------------------------
\*                                                                         */

void init_name_list(      string     list[],
                    const matrix_dim size) 
{
   register matrix_dim i;

   for(i=0; i < size; i++) {
      list[i] = NULL;
   }
} /* init_name_list() */


void free_name_list(      string     list[],
                    const matrix_dim maxsize) 
{
   register matrix_dim i;

   for(i=0; (i < maxsize) && (list[i]); i++) {
      free(list[i]);
   }
} /* free_name_list() */


bool make_name_list(      string    list[],
                    const chose_t * source)
{
            char   proc[] = "make_name_list";
   register elem_t item;
   register elem_t count;
            elem_t expected = s_count(source);

   for (item=count=0; (item < source->size) && (count < expected); item++) {
      if (ba_value(source->selected, item)) {
         if (NULL != list[count]) {
            free(list[count]);
	 }
         if (1 == source->size) {
            list[count++] = NULL;
	 } else {
            if (NULL == (list[count++] = strdup(source->name[item]))) {
               return (FALSE);
	    }
	 }
      }
   } /* for i */
   if (count < expected) {
      extern char errorstring[];

      sprintf(errorstring, "Could only find "ELEM_SPEC" of "ELEM_SPEC \
                           " names!",                                 \
                           count, expected);
      swerr(proc, errorstring, 0);
      return(FALSE);
   }
   return (TRUE);
} /* make_name_list() */


bool print_name_list(const string     name_list[],
                     const elem_t     maxsize,
                           ScreenSize indent,
                     const ScreenSize maxwidth,
                     const string     title,
                           FILE *     outfile) 
{
   register elem_t     i,
                       this_line = 0;
            ScreenSize width = 0;
            bool       status = TRUE;
            size_t     currwidth;

   if (indent > maxwidth) {
      indent = 0;
   }
   if (title) {
      status = (status && (EOF != fputs(title, outfile)));
      status = status && NewLine(1, outfile);
   }
   for (i=0; (i < maxsize) && name_list[i]; i++) {
      if (0 == this_line) { /* indent start of line */
         status = status && (EOF != fprintf(outfile, "%*s", indent, ""));
         width = indent;
      }

      currwidth = strlen(name_list[i]);
      if ((maxwidth > (currwidth + width + 2)) || (0 != this_line)) {
         status = status &&                  \
            (EOF != fprintf(outfile, "%s%s", \
                    name_list[i], (maxsize - 1 != i)?", " :""));
         width += (currwidth + 2);
         this_line++;
      }
      if ((maxwidth <= (currwidth + width + 2)) || (maxsize - 1 == i)) {
         status = status && NewLine(1, outfile);
         this_line = 0;
      }
   }
   return(status);
} /* print_name_list() */


void make_level_list(      char      level_list[],
                     const chose_t * source,   /* which levels are included */
                     const string    level_name[]) 
{
   register matrix_dim level;
            bool       first;

   level_list[0] = '\0';
   for (first=TRUE,level=0; level < source->size; level++) {
      if (ba_value(source->selected, level)) {
         if (first) {
            first = FALSE;
         } else {
            strcat(level_list, ", ");
	 }
         strcat(level_list, level_name[level]);
      }
   }
} /* make_level_list() */
